package Narada::AjaxAccountHandler;

use strict;
use warnings;
use JSON::XS;

use Apache2::Const -compile => qw(OK );
use Digest::MD5 qw(md5_hex);
use Narada;


sub handler {
    my ($r)= @_;
    my $msg = '';
    my $err = 0;

    my $narada = new Narada({ r => $r});
    my $form = $narada->getForm();

    # obtain the session
    my $sessionref= $narada->getSession();

    # if the user is logged in, they do not need to create an account
    if ($sessionref->{uid}) {
        $err = -1;
    }
    else {

        if ($r->method() eq 'POST') {
            if ($form->{password} ne $form->{verify}) {
                $msg= "Password and password verify do not match";
                $err= 1;
            }

            my $uid= $narada->userExists({ 
                username    => $form->{username},
                email       => $form->{email}});

            warn "uid $uid" if $uid;

            if ($uid) {
                $msg = <<EOMSG;
The user $form->{username} or email $form->{email} already exist.
Please try another value.";
EOMSG
                $err= 1;
            }
            else {
                warn "attempting to saveUser $form->{username}";
                my $saved = $narada->saveUser({
                    username    => $form->{username},
                    email       => $form->{email},
                    password    => md5_hex($form->{password}),
                    firstname   => $form->{firstname},
                    surname     => $form->{surname},
                    phone       => $form->{phone},
                    state       => $form->{state},
                    city        => $form->{city},
                    address     => $form->{address} });

                if ($saved) {
                    notifyUser($form);
                    $msg = <<EOMSG;
The account for $form->{username} has been created 
and information sent to $form->{email}
EOMSG
                }
                else {
                    $msg = "There was a problem creating the account for $form->{username}";
                }

            }
        }
    }

    $r->content_type('text/html');
    $r->print(encode_json({ 
            msg => $msg,
            err => $err }));

    return Apache2::Const::OK;

}

sub notifyUser {
    my ($form) = @_;
    my $username = $form->{username};
    my $email = $form->{email};
    my $password = $form->{password};
    my $login_url = 'http://patg.net/login';

    warn "sending mail for $username $email";
    my $subject= "New Account and Password information";
    my $msg = <<EOMSG;
A new account has been set up for $form->{username}
 with the email address $form->{email} 

Username: $username
Password: $password

Now you can log in at: $login_url.

EOMSG

    sendEmail($email, 'accounts@patg.net', $subject, $msg, 'patg.net');
}

1;
