package Narada::LoginHandler;

use strict;

use Apache2::Const -compile => qw(OK REDIRECT);
use Apache2::Request;
use Data::Dumper;
use Narada;

my $url;

#
# main 
#   run all other primary display subroutines 
#
#   arguments:
#    none 
# 
#   returns:
#     nothing 
#
sub handler {
    my ($r)= @_;
    my $msg;

    $url= $r->uri;
    my $narada= new Narada({ r => $r});
    my $form= $narada->getForm();

    # this is the URL that the user came from, so when they are
    # given a session, they will return to that page 
    my $returnto= $form->{returnto};

    my $path_info= substr($r->path_info, 1, length $r->path_info);
    my $headers= $r->headers_in();

    # obtain the session
    my $sessionref= $narada->getSession(\$msg);

    # if they are storing session values
    if ($form->{save}) {
        $sessionref= $narada->storeSession($sessionref->{sessionid},
                { $form->{fieldname} => $form->{fieldvalue}});
    }
    
    # if the user has a session id and the returnto value is set
    # then redirect them back to that page
    if ($sessionref->{sessionid}) {
        if ($returnto) {
            $r->headers_out->add('Location' => $returnto);
            return Apache2::Const::REDIRECT;
        }

    }

    $r->content_type('text/html');
    $r->print(header());
    $r->print(body($sessionref, $returnto, $msg));
    $r->print(footer());

    return Apache2::Const::OK;
}

#
# header 
#   prints the header of the page, including style sheet 
#
#   arguments:  none
# 
#   returns:
#     HTML code
#
sub header {

    return <<EOHTML;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
 "http://www.w3.org/TR/html4/strict.dtd">
<html>
  <head>
    <title>Session Example</title>
    <link type="text/css" href="/css/narada.css" rel="stylesheet">
 </head>
  <body>
  <!-- end of header() content -->
EOHTML

}

#
# footer 
#   prints the footer of the page
#
#   arguments:  none
# 
#   returns:
#     HTML code
#
sub footer {
  return <<EOHTML; 
<!-- footer -->
  </body>
</html>
EOHTML
}

sub body {
    my ($sessionref, $returnto, $msg) = @_;
    warn $returnto;
    my $data= '';

    # default messages
    # leave room for unauthenticated sessions (those without uid)
    $msg||= ($sessionref->{sessionid} && $sessionref->{uid}) ? 
        "User UID $sessionref->{uid} is logged in" :
        "Login Form";

    # if there is a session and user is logged in
    if ($sessionref->{sessionid}) {
        $data= sessionInfo($sessionref, $msg);
    }
    # if there is not a session, user is not logged in
    else {
        $data= loginForm($returnto, $msg);
    }

    # handy reload to test page
    $data.= qq(\t\t<p><a href="$url">Reload page</a></p>\n);
    return $data;
}

sub sessionInfo {
    my ($sessionref, $msg)= @_;

    my $table= sessionVariablesTable($sessionref);

    return <<EOPAGE;
        <p class="msg">$msg</p>
        <!-- begin loginform -->
        <p>
        <form action="$url" id="loginform" name="loginform" method="post">
            <p>Add a session field and value:</p>
            <fieldset>
                <label>Field Name</label>
                <p><input type="text" name="fieldname"></p>
                <label>Field Value</label>
                <p><input type="text" name="fieldvalue"></p>
                <input type="submit" name="save" value="Save">
            </fieldset>
        </form>
        </p>
        $table
        <p>
        <form name="logout" method="POST" action="$url">
        <fieldset><input type="submit" name="logout" value="Log out"></form></fieldset>
        </p>
        <!-- end loginform -->
EOPAGE

}

sub sessionVariablesTable {
    my ($sessionref)= @_;
    my $table='';

    # if the session has values, display what they are
    if (scalar keys %$sessionref) {
        $table .= <<EOTABLE;
        <p class="msg">Session data for session $sessionref->{sessionid} UID $sessionref->{uid}</p>
        <table class="userlist">
            <thead>
            <tr><th>Key</th><th>Value</th></tr>
            </thead>
            <tbody>
EOTABLE

        for my $key (keys %$sessionref) {
            my $value;
            if (ref($sessionref->{$key})) { 
                $value= Dumper $sessionref->{$key};
                $value= '<pre>' . $value . '</pre>' ;
            }
            else {
                $value= $sessionref->{$key}; 
            }

            $table .=" 
                <tr><td>$key</td><td>$value</td></tr>\n";
        }

        $table .="
            </tbody>
            </table>
            </p>\n";
    }

    return $table;
}

sub loginForm {
    my ($returnto, $msg)= @_;
    if ($returnto) {
        $returnto = qq(<input type="hidden" name="returnto" value="$returnto">);
    }
    return <<EOPAGE;
    <!-- begin loginform -->
    <p class="msg">$msg</p>
    <p>
    <form action="$url" id="loginform" name="loginform" method="post">
      <fieldset>
        <label>Username</label>
        <p>
          <input type="text" name="username">
        </p>
        <label>Password</label>
        <p>
          <input type="password" name="password">
        </p>
        $returnto
        <input type="submit" name="login" value="Login">
    </form>
    </p>
    <p class="msggrey">Don't have an account yet? <a href="/account"> Signup!</a></p>
    <!-- end loginform -->
EOPAGE
}
1;
