package Narada::URLHandler;

use strict;

use Apache2::Const -compile => qw(OK REDIRECT SERVER_ERROR);
use Apache2::Request;
use Template; 
use Getopt::Long;
use Digest::MD5 qw(md5_hex);
use Narada;


sub handler {
    my ($r)= @_;
    my $msg;
    my $url= $r->uri;
    my $template_path;
    my $sresults;

    my $narada= new Narada({ r => $r});

    my $form= $narada->getForm();

    my $sessionref= $narada->getSession(\$msg);

    # if no session, redirect to login page
    unless ($sessionref && $sessionref->{uid}) {
        $r->headers_out->add('Location' => "/login?returnto=$url");
        return Apache2::Const::REDIRECT;
    }

    $template_path= $r->dir_config('TEMPLATE_PATH');

    # obtain the list of urls in urls_queue
    my $urls = $narada->dbGetRef('urls_queue', 'url,url_md5');

    # instantiate a new Template object
    my $template = Template->new({
        INCLUDE_PATH => $template_path, 
        POST_CHOMP   => 1,            
    });

    # set up template parameters
    my $tparams = {
        msg         => "Session " . $sessionref->{sessionid} . " logged in",
        url         => $r->uri,
        urls        => $urls,
    };

    $url =~ s/^\///;
    my $tname= $url . '.tt2';
    my $template_cached= $narada->cacheTemplate($template_path,$tname); 

    if ($template_cached) {
        $tname = \$template_cached;
    }

    $r->content_type('text/html');

    $template->process($tname, $tparams) ||
        do {
            # if an error, return SERVER_ERROR
            $r->server->warn($template->error());
            return Apache2::Const::SERVER_ERROR;
        };

    return Apache2::Const::OK;
}

1;
