# named to whatever naming scheme you need 
# use this with ajaxform1_proto.html
package Narada::URLQueueHandler;

# good practice
use strict;
use warnings;
use JSON::XS;

# list of return values you want to use
use Apache2::Const -compile => qw(OK );
use Digest::MD5 qw(md5_hex);
use Gearman::Client;
use Narada;


sub handler {
    my ($r)= @_;
    my $msg = '';
    my $err = 0;
    my $updated = 0;

    my $narada = new Narada({ r => $r});
    my $gman_client = Gearman::Client->new;

    # narada->{gman_servers} is an array reference, need to dereference
    $gman_client->job_servers(@{$narada->{gman_servers}});

    my $form = $narada->getForm();

    my $url_md5= $form->{url_md5};

    $url_md5 ||= $form->{url} ? md5_hex($form->{url}) : '';

    # obtain the session
    my $sessionref= $narada->getSession();

    # if the user is logged in, the javascript function will redirect them
    # to log in
    unless ($sessionref->{uid}) {
        $err = -1;
    }
    else {
        if ($r->method() eq 'POST') {
            if ($form->{create}) {
                if ($form->{url}) {
                    my $urls = $narada->dbGetRef('urls_queue',
                        'url,url_md5', { url_md5 => $url_md5});

		    print STDERR "calling dispatch_background('url_fetch', $form->{url})\n";

		    my $gman_ret = $gman_client->dispatch_background('url_fetch', $form->{url});
		    print STDERR "gman_ret: $gman_ret\n";

                    if (scalar @$urls) {
                        $narada->dbUpdate('urls_queue',
                            { '-last_updated'  => 'now()'},
                            { url_md5       => $url_md5 });

                        $msg = "Updated $form->{url}";
                        $updated = 1;
                    }
                    else {
                        $narada->dbInsert('urls_queue',{
                            url         => $form->{url},
                            url_md5     => $url_md5 }
			); 

                        $msg = "Inserted $form->{url} into the queue";
                    }
                }
                else {
                    $msg = "Missing URL value";
                    return 1;
                }
            }
            elsif ($form->{'delete'}) {
                my $deleted= $narada->dbDelete('urls_queue',
                    { url_md5 => $url_md5});
                $msg = "Deleted $url_md5 from the queue";
            }
            elsif ($form->{'update'}) {
		$gman_client->dispatch_background('url_fetch', $form->{url});
                my $updated= $narada->dbUpdate('urls_queue',
                    { '-last_updated' => 'now()'},
                    { url_md5 => $url_md5 });
                $msg = "Updated $form->{url}";
                $updated = 1;
            }

        }
    }

    $r->content_type('text/html');
    $r->print(encode_json({ 
            url_md5 => $url_md5,
            msg     => $msg,
            url     => $form->{url},
            updated => $updated,
            err     => $err }));

    return Apache2::Const::OK;

}

1;
