#!/usr/bin/perl

use Sphinx::Search;
use strict;
use Data::Dumper;
use Getopt::Long;

our $opt_term;
our $opt_index;
our $opt_verbose;

GetOptions( 'v|verbose' => \$opt_verbose,
            'i|index=s' => \$opt_index,
            't|term=s'  => \$opt_term) or usage();

$opt_term or usage();
$opt_index ||= 'dist_urls';

my $sphinx = Sphinx::Search->new();


$sphinx->SetServer('localhost', 9312); 
$sphinx->SetRankingMode(SPH_RANK_PROXIMITY_BM25);
$sphinx->SetLimits(0, 1000);

# other options for extending functinality
$sphinx->SetMatchMode(SPH_MATCH_EXTENDED2);
# this provides ordering, but ordering is lost from the db.
#$sphinx->SetSortMode(SPH_SORT_EXTENDED,'ulast_updated') ;

my $hits= $sphinx->Query($opt_term, $opt_index);
my $warning = $sphinx->GetLastWarning;
print STDERR Dumper $hits;
print "warning $warning\n" if $opt_verbose;
my $error = $sphinx->GetLastError;

print "error: $error\n";

sub usage {
    print <<EOUSAGE;
usage: search_test.pl [options]

-v or --verbose                     Verbose
-i index or --index=index name      Index name
-t term or --term=term              Search term
EOUSAGE
    exit(0);
}
