#!/usr/bin/perl
#
use strict;

use Getopt::Long;
use Digest::MD5 qw(md5_hex);
use Gearman::Worker;
use Gearman::Client;
use Narada;
my $narada= new Narada();

use NaradaConf;

sub url_process {
    my $count= $narada->{memc}->get('urls_stored');
    warn "Count $count\n";
    store_urls() if $count >= $URL_STORE_COUNT;
    return;
}

sub store_urls {
    my $pageref;
    my $gman_client = Gearman::Client->new;
    $gman_client->job_servers(@{$narada->{gman_servers}});

    $narada->{dbh}->do('BEGIN WORK');
    my $urls = $narada->dbGetRef('urls_stored', 'url, url_md5');
    unless (scalar @$urls) {
        $narada->{dbh}->do('COMMIT');
        return;
    }

    my $where_in = ' WHERE url_md5 IN ( ';
    $where_in .= join(',', map { $narada->{dbh}->quote($_->{url_md5}) } @$urls);
    $where_in .= ') ';

    $narada->dbDelete('urls_stored', $where_in);
    
    $narada->{dbh}->do('COMMIT');
    my $pages_exist = {};
    my $pref= $narada->dbGetRef('urls', 'url_md5,id', $where_in);
    map { $pages_exist->{$_->{url_md5}}= $_->{id}} @$pref;


    for my $url (@$urls) {
        my $url_md5 = $url->{url_md5};
        my $doc_ref;
        $doc_ref->{url}= 	$narada->{memc}->get("$url_md5:url");
        $doc_ref->{status}= 	$narada->{memc}->get("$url_md5:status");
        $doc_ref->{status}||= '';
        $doc_ref->{title}= 		$narada->{memc}->get("$url_md5:title");
        $doc_ref->{last_modified}= 	$narada->{memc}->get("$url_md5:last_modified");
        $doc_ref->{last_modified}||= '';
        $doc_ref->{content}= 	$narada->{memc}->get("$url_md5:content");

        my $file_id= $pages_exist->{$url_md5};

        if ($file_id) {
            print "update $url_md5\n" if $DEBUG;
            $narada->dbUpdate('urls, urls_blob', $doc_ref,
                { '-urls.id'            => 'urls_blob.id',
                'urls.id'               => $file_id });
        }
        else {
            print "insert $url_md5\n" if $DEBUG;
            my $id= $narada->dbInsert('urls', {
                'url_md5'       => $url_md5,
                'title'         => $doc_ref->{title}, 
                'url'           => $doc_ref->{url}, 
                'status'        => $doc_ref->{status}, 
                'last_modified' => $doc_ref->{last_modified}, 
                'created'       => 'now()'
                }); 

            $narada->dbInsert('urls_blob', {
                id          => $id,
                'content'   => $doc_ref->{content}, 
                });
        }
        $narada->{memc}->incr('index_counter');
    }
# reset the counter
    $narada->{memc}->set('urls_stored', 0);
# request indexer worker
    $gman_client->dispatch_background('indexer', 'http://blank.url');
    return;
}


my $worker= Gearman::Worker->new();
$worker->job_servers(@{$narada->{gman_servers}});
$worker->register_function('url_process',
    \&url_process);
$narada->{memc}->set('index_counter', 1);
$worker->work() while 1;

