#!/usr/bin/perl
#
use strict;

use Getopt::Long;
use Digest::MD5 qw(md5_hex);
use Gearman::Worker;
use Narada;
my $webapp= new Narada();

use NaradaConf;

sub url_process {
    my $count= $webapp->{memc}->get('urls_stored');
    warn "Count $count\n";
    store_urls() if $count >= $URL_STORE_COUNT;
    return;
}

sub store_urls {
    my $pageref;

    $webapp->{dbh}->do('BEGIN WORK');
    my $urls = $webapp->dbGetRef('urls_stored', 'url, url_md5');
    unless (scalar @$urls) {
        $webapp->{dbh}->do('COMMIT');
        return;
    }

    my $where_in = ' WHERE url_md5 IN ( ';
    $where_in .= join(',', map { $webapp->{dbh}->quote($_->{url_md5}) } @$urls);
    $where_in .= ') ';

    $webapp->dbDelete('urls_stored', $where_in);
    
    $webapp->{dbh}->do('COMMIT');
    my $pages_exist = {};
    my $pref= $webapp->dbGetRef('urls', 'url_md5,id', $where_in);
    map { $pages_exist->{$_->{url_md5}}= $_->{id}} @$pref;


    for my $url (@$urls) {
        my $url_md5 = $url->{url_md5};
	my $doc_ref;
	$doc_ref->{url}= 	$webapp->{memc}->get("$url_md5:url");
	$doc_ref->{status}= 	$webapp->{memc}->get("$url_md5:status");
	$doc_ref->{status}||= '';
	$doc_ref->{title}= 		$webapp->{memc}->get("$url_md5:title");
	$doc_ref->{last_modified}= 	$webapp->{memc}->get("$url_md5:last_modified");
	$doc_ref->{last_modified}||= '';
	$doc_ref->{content}= 	$webapp->{memc}->get("$url_md5:content");

        my $file_id= $pages_exist->{$url_md5};

        if ($file_id) {
            print "update $url_md5\n";
            $webapp->dbUpdate('urls, urls_blob', $doc_ref,
		{ '-urls.id'            => 'urls_blob.id',
                'urls.id'               => $file_id });
        }
        else {
            print "insert $url_md5\n";
            my $id= $webapp->dbInsert('urls', {
               'url_md5'       => $url_md5,
               'title'         => $doc_ref->{title}, 
               'url'           => $doc_ref->{url}, 
               'status'        => $doc_ref->{status}, 
               'last_modified' => $doc_ref->{last_modified}, 
	       'created'       => 'now()'
                }); 

            $webapp->dbInsert('urls_blob', {
                id          => $id,
                'content'   => $doc_ref->{content}, 
                });
        }
	$webapp->{memc}->incr('index_counter');
    }
    # reset the counter
    $webapp->{memc}->set('urls_stored', 0);
    # request indexer worker
    $webapp->{dbh}->do('set @indexer = gman_do_background(\'indexer\',\'http://blank.url\')');
    return;
}


my $worker= Gearman::Worker->new();
$worker->job_servers($webapp->{gman_servers});
$worker->register_function('url_process',
                           \&url_process);
$worker->work() while 1;

