#!/usr/bin/perl
#
use strict;
use warnings;

use LWP;
use HTTP::Request;
use Getopt::Long;
use HTML::Strip;
use Digest::MD5 qw(md5_hex);
use Gearman::Worker;
use Encode qw(encode_utf8);
use Narada;
use Data::Dumper;

our $recurse= 3;
my $webapp= new Narada();

sub url_fetch {
    fetch_remote_doc($_[0]->arg, $recurse);
}

sub fetch_remote_doc {
    my ($local_url)= @_;
    warn "processing url $local_url\n";

    my $last_modified; 
    my $if_modified= get_if_modified($local_url);

    # Create a user agent object
    use LWP::UserAgent;
    my $ua = LWP::UserAgent->new;

    $ua->agent("TofuSearchClient/0.1 ");

    # Create a request
    my $req = HTTP::Request->new(GET => $local_url);

    my $res = $ua->request($req); #, 'If-Modified-Since' => $if_modified);

    if ($res->is_success) {
        my $html= $res->content;

        my $title= parse_title($html);
        $title ||= $local_url;

        my $urls = parse_links($html);

        my $hs = HTML::Strip->new();
        my $plain_text = $res->content ? $hs->parse( $res->content) : '';
        $hs->eof;

        $plain_text =~ s/\s{2,}/ /g;
        $plain_text =~ s/\W/ /g;

        $last_modified ||= $res->header('Last-Modified'); 
        $last_modified ||= '';

        my $content_type= $res->header('Content-type');
        if ($content_type =~ /^text\//) {
            my $pageref = {
                'last_modified' => $last_modified,
                'title'         => $title,
                'url'           => $local_url,
                'content'       => $plain_text,
                'status'        => $res->status_line,
            };

            store_page($pageref);
        }

        while ($recurse) {
		print Dumper $urls;
	    print "RECURSE: $recurse\n";
            $recurse--;
            for my $murl (keys %$urls) {
                fetch_remote_doc($murl);
            }
        }
    }

}

sub store_page {
    my ($pageref)= @_;

    my $url_md5= md5_hex(encode_utf8($pageref->{url}));
    warn "trying to store $url_md5 $pageref->{url}\n";

    #$webapp->{dbh}->do("SELECT memc_servers_set('127.0.0.1:11211')");
    $webapp->{dbh}->do("SELECT gman_servers_set('127.0.0.1:4730')");
    my $exists= $webapp->dbGetRef('urls_stored', 'url_md5', { url_md5 => $url_md5});

    unless (scalar @$exists) {
        warn "storing $url_md5:title\n";

        $webapp->{memc}->set("$url_md5:url", encode_utf8($pageref->{url})); 
        $webapp->{memc}->set("$url_md5:status", encode_utf8($pageref->{status})); 
        $webapp->{memc}->set("$url_md5:title", encode_utf8($pageref->{title})); 
        $webapp->{memc}->set("$url_md5:last_modified", encode_utf8($pageref->{last_modified})); 
        $webapp->{memc}->set("$url_md5:content", encode_utf8($pageref->{content})); 

        $webapp->dbInsert('urls_stored', { 
            url         => $pageref->{url},
            url_md5     => $url_md5,
            });
    }

}

sub get_if_modified {
    my ($url)= @_;

    my $uref= $webapp->dbGetRef('urls', 'last_modified', {url => $url});
    return $uref->[0]{last_modified};
}

sub is_junk {
    my ($url)= @_;
    return 1 if $url =~ /viagra/i;
    return 0;
}

=head2 parse_title

Parses the title from HTML

=item Args

$html  HTML document

=item returns 

title text

=cut
sub parse_title {
    my ($html)= @_;

    my   ($title) = $html =~ /\<title\>([\w\W\r\n]+)\<\/title\>/gi;
    $title =~ s/\r|\n|//g;
    $title =~ s/\s{2,}/ /g;

    return $title;
}

sub parse_links {
    my ($html)= @_;
    my $urls;
    my @doc= split("\n", $html);
    for my $line(@doc) {
        my @url_list= $line =~ /href=\"([^"]+)\"/gi;
        for my $url(@url_list) {
            is_junk($url) or $urls->{$1}++;
        }
    }
    return $urls;
}

my $worker= Gearman::Worker->new();
$worker->job_servers('127.0.0.1:4730');
$worker->register_function('url_fetch',
                           \&url_fetch);
$worker->work() while 1;

