#!/usr/bin/perl
#
use strict;

use Getopt::Long;
use Digest::MD5 qw(md5_hex);
use Gearman::Worker;
use Narada;
my $webapp= new Narada();

sub url_process {
    $webapp->{dbh}->do("SELECT gman_servers_set('127.0.0.1:4730')");
    my $count= $webapp->{memc}->get('urls_stored');
    warn "Count $count\n";
    store_urls();
}

sub store_urls {
    my $pageref;

    $webapp->{dbh}->do('BEGIN WORK');
    my $urls = $webapp->dbGetRef('urls_stored', 'url, url_md5');
    unless (scalar @$urls) {
        $webapp->{dbh}->do('COMMIT');
        return;
    }

    my $where_in = ' WHERE url_md5 IN ( ';
    $where_in .= join(',', map { $webapp->{dbh}->quote($_->{url_md5}) } @$urls);
    $where_in .= ') ';

    $webapp->dbDelete('urls_stored', $where_in);
    
    $webapp->{dbh}->do('COMMIT');
    my $pages_exist = {};
    my $pref= $webapp->dbGetRef('urls', 'url_md5,id', $where_in);
    map { $pages_exist->{$_->{url_md5}}= $_->{id}} @$pref;

    for my $url (@$urls) {
        my $url_md5 = $url->{url_md5};

        my $file_id= $pages_exist->{$url_md5};

        if ($file_id) {
            print "update $url_md5\n";
	    $webapp->{dbh}->do("select memc_servers_set('localhost:11211,webserver:11211')");
            $webapp->dbUpdate('urls, urls_blob', {
                '-status'           => "memc_get('$url_md5:status')",  
                '-title'            => "memc_get('$url_md5:title')",
                '-last_modified'    => 'now()',
                '-content'          => "memc_get('$url_md5:content')" },
                { '-urls.id'            => 'urls_blob.id',
                'urls.id'               => $file_id });
        }
        else {
            print "insert $url_md5\n";
	    $webapp->{dbh}->do("select memc_servers_set('localhost:11211,webserver:11211')");
            my $id= $webapp->dbInsert('urls', {
               url_md5          => $url_md5,
               '-title'         => "memc_get('$url_md5:title')",
               '-url'           => "memc_get('$url_md5:url')", 
               '-status'        => "memc_get('$url_md5:status')", 
               '-last_modified' => 'now()', 
                '-created'      => 'now()'
                }); 

            $webapp->dbInsert('urls_blob', {
                id          => $id,
                '-content'  => "memc_get('$url_md5:content')"
                });
        }
    }
}


my $worker= Gearman::Worker->new();
$worker->job_servers('127.0.0.1:4730');
$worker->register_function('url_process',
                           \&url_process);
$worker->work() while 1;

