<?php

# Narada Search Engine in PHP
# Copyright (C) 2009 Eric Day, Patrick Galbraith
# All rights reserved.
#
# Use and distribution licensed under the BSD license.  See
# the COPYING file in this directory for full text.

# This is the Gearman worker that fetches URLs, extracts more URLs, and stores
# the document into the database.

require('Narada.php');

class NaradaFetch extends Narada
{
  private $crawl;
  private $local;

  public function __construct()
  {
    parent::__construct();
    $this->functionName= $this->fetchFunctionName;
    $this->crawl= $GLOBALS['NaradaFetchCrawl'];
    $this->local= $GLOBALS['NaradaFetchLocalURLs'];
  }

  public function callback($job)
  {
    $url= $this->get($job->workload());

    $this->debug("Fetching: $url");

    list($protocol, $domain, $path, $file)= $this->split($url);

    $this->debug("Protocol: $protocol");
    $this->debug("Domain: $domain");
    $this->debug("Path: $path");
    $this->debug("File: $file");

    # Fetch document.
    $doc = new DomDocument();
    @$doc->loadHTMLFile($url);

    if ($this->crawl)
    {
      # Extract the URLs from document object.
      $xpath = new DomXpath($doc);
      $res = $xpath->query("//a/@href");
      $new_url_list= array();
      foreach($res as $href)
      {
        if (strstr($href->nodeValue, "mailto:"))
          continue;
        else if (strstr($href->nodeValue, "://"))
          $new_url= $href->nodeValue;
        else if (substr($href->nodeValue, 0, 1) == "#")
          continue;
        else if (substr($href->nodeValue, 0, 1) == "/")
          $new_url= $domain . $href->nodeValue;
        else
          $new_url= $domain . $path . $href->nodeValue;

        # Remove local references.
        if (strstr($new_url, "#"))
          $new_url= substr($new_url, 0, -strlen(strstr($new_url, "#")));

        $new_url= $this->verify($new_url);
        if ($new_url)
          $new_url_list[]= $new_url;
      }

      # Remove duplicate URLs.
      $url_list= array_unique($new_url_list);

      # Filter URL list into local and remote URLs.
      foreach ($url_list as $new_url)
      {
        if (strstr($new_url, $domain))
        {
          $this->debug("Local URL: $new_url");
          $this->insert($new_url);
        }
        else
        {
          if ($this->local)
            $this->debug("Skipping Remote URL: $new_url");
          else
          {
            $this->insert($new_url);
            $this->debug("Remote URL: $new_url");
          }
        }
      }
    }

    $nodes= $doc->getElementsByTagName('title');
    $title= '';
    foreach ($nodes as $node)
      $title .= $node->nodeValue;

    $this->debug("Title: $title");
    #$this->debug("########################################");
    #$this->debug($doc->textContent);
    #$this->debug("########################################");

    # Store the document in the database.
    $this->storeContent($job->workload(), $title, $doc->textContent);

    # Send a request to the indexer.
    $this->index();
  }

  private function split($url)
  {
    $path= strstr(substr($url, 8), "/");
    $protocol= substr($url, 0, -(strlen(strstr($url, "://")) - 3));
    $domain= substr($url, strlen($protocol), -strlen($path));
    $file= substr(strrchr($path, "/"), 1);
    if (strlen($file) > 0)
      $path= substr($path, 0, -strlen($file));

    return array($protocol, $domain, $path, $file);
  }
}

# Now, actually run the fetch worker.
$fetch= new NaradaFetch();
$fetch->runWorker();

?>
